/**
 * @module ctt
 * @submodule direct-contents-transfer-controller
 */
(function(){
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var ModalDialogView = dixim.ctt.ModalDialogView;
var TransferWizardController = dixim.ctt.TransferWizardController;
var TransferStatusMonitor = dixim.ctt.TransferStatusMonitor;
var TransferProgressController = dixim.ctt.TransferProgressController;
var Template = dixim.ctt.util.Template;
var $ = dixim.ctt.jQuery;
var _ = dixim.ctt.ResourceBundle.get;

var DirectContentsTransferController;
var prototype;

/**
 * Root controller of the Direct Contents Transfer.
 *
 * @class DirectContentsTransferController
 * @constructor
 * @param {Object} configs config object.
 */
DirectContentsTransferController = function(configs) {
  this._initialize(configs);
};
dixim.ctt.DirectContentsTransferController = DirectContentsTransferController;
prototype = DirectContentsTransferController.prototype;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs config object.
 */
prototype._initialize = function(configs) {
  this._pin = null;
  this._session = null;
  this._modal = null;
  this._indicator = null;

  this._api = new ContentsTransferApiClient();
  this._transferStatusMonitor = new TransferStatusMonitor({model: this._api});
};

prototype._showActivityIndicator = function() {
  var loading;

  if (!!this._indicator) return;
  loading = $(Template.LOADING_OVERLAY).appendTo('body');
  this._indicator = loading;
};

prototype._hideActivityIndicator = function() {
  if (!!this._indicator) {
    this._indicator.remove();
    this._indicator = null;
  }
};

/**
 * @method start
 */
prototype.start = function() {
  this._showActivityIndicator();
  this._transferStatusMonitor.addEventListener('fetch', this._didFetchState, this);
  this._transferStatusMonitor.start();
};

prototype._fetchRequestedTransferList = function() {
  if (!!this._session) return;
  this._session = this._api.getRequestedTransferList();
};

prototype._didFetchState = function(e) {
  var monitor = this._transferStatusMonitor;
  var api = this._api;

  monitor.removeEventListener('fetch', this._didFetchState, this);
  monitor.addEventListener('transferring', this._didEnterTransferringState, this);
  monitor.addEventListener('canceling', this._didEnterTransferringState, this);

  if (e.args.done) {
    api.addEventListener('getRequestedTransferList', this._didFetchReqeustedTransferList, this);
    api.addEventListener('getRequestedTransferListError', this._didFailFetchReqeustedTransferList, this);
    this._fetchRequestedTransferList();
  }
};

prototype._didFetchReqeustedTransferList = function(e) {
  var args = e.args;
  var results = args.data.results;
  var session = args.session;
  var item;
  var i;
  var len;

  if (this._session !== session) return;
  this._session = null;
  if (session.isCancelled()) return;

  this._hideActivityIndicator();

  if (!(len = results.length)) {
    this._showClosePrompt(_('ctt_direct_contents_transfer_dialog_close_prompt_no_media_available_html'));
    return;
  }

  for (i = 0; i < len; i += 1) {
    item = results[i];
    if (item['class'] !== 'item' || !!!item['profile'] || !item['uploadable']) {
      this._showClosePrompt(_('ctt_direct_contents_transfer_dialog_close_prompt_unsupported_media_html'));
      this._cancelTransfer();
      return;
    }
  }

  this._transferTargets(results);
};

prototype._didFailFetchReqeustedTransferList = function(e) {
  var session = e.args.session;

  if (this._session !== session) return;
  this._session = null;
  this._hideActivityIndicator();
  this._showClosePrompt(_('ctt_direct_contents_transfer_dialog_close_prompt_no_media_available_html'));
};

prototype._transferTargets = function(items) {
  var wizard;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  wizard = new TransferWizardController({
    targetItems: items,
    model: this._api,
    enableWol: false,
    enableScheduledTransferring: false
  });
  wizard.addEventListener('done', this._doneTransferWizard, this);
  wizard.addEventListener('cancel', this._didCancelTransferWizard, this);
  wizard.start();
  this._modal = wizard;
};

prototype._didEnterTransferringState = function(e) {
  this._hideActivityIndicator();
  this._startTransferProgress();
};

prototype._doneTransferWizard = function(e) {
  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  this._startTransferProgress();
};

prototype._startTransferProgress = function() {
  var vc;

  this._transferStatusMonitor.stop();

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferProgressController({
    manualCancel: true,
    closable: false
  });
  vc.addEventListener('requestCancel', this._didRequestCancelTransferProgress, this);
  vc.start();

  this._modal = vc;
};

prototype._didRequestCancelTransferProgress = function(e) {
  var vc = e.args.controller;
  var dialog;

  dialog = new ModalDialogView({
    title: _('ctt_transfer_cancellation_dialog_title'),
    contentBox: ['<div class="ctt-transfer-cancellation-dialog-description">',
                 _('ctt_transfer_cancellation_dialog_description_html'),
                 '</div>'].join(''),
    width: 320,
    height: 'auto',
    showCloseButton: false
  });

  dialog.setButtons([
    {
      text: _('ctt_transfer_cancellation_dialog_button_label_continue'),
      click: function() {
        dialog.destroy();
      },
      align: 'left'
    },
    {
      text: _('ctt_transfer_cancellation_dialog_button_label_cancel'),
      click: function() {
        dialog.destroy();
        vc.cancel();
      }
    }
  ]);

  vc.addEventListener('done', function(e) {
    dialog.destroy();
  }, this);

  dialog.show();
};

prototype._showClosePrompt = function(message) {
  var dialog;

  dialog = new ModalDialogView({
    contentBox: ['<div class="ctt-direct-contents-transfer-dialog">',
                   '<div class="ctt-direct-contents-transfer-dialog-close-prompt">',
                     message,
                   '</div>',
                 '</div>'].join(''),
    title: _('ctt_direct_contents_transfer_dialog_title'),
    width: 360,
    height: 'auto',
    minHeight: 0,
    showCloseButton: false
  });
  dialog.show();
};

prototype._didCancelTransferWizard = function(e) {
  this._dismissWizard();
  this._cancelTransfer();
  this._showClosePrompt(_('ctt_direct_contents_transfer_dialog_close_prompt_finished_with_cancellation_html'));
};

prototype._dismissWizard = function() {
  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
};

prototype._cancelTransfer = function() {
  if (!!this._session) {
    this._session.cancel();
  }
  this._api.cancelTransfer();
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._transferStatusMonitor.stop();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
};

}());
